/*
 * Graph.cpp	(CCFlib)
 * Copyright (C) 2004 Kento EMOTO
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Kento EMOTO:
 *   mailto:emoto@ipl.t.u-tokyo.ac.jp
 *   http://www.ipl.t.u-tokyo.ac.jp/~emoto/
 */
/** 
 *	@file Graph.cpp implementation of functions of Graph.
 *	@author Kento EMOTO
 */
#include "Graph.hpp"

/**
 *	constructor.
 *	initially this graph consists of no vertices and edges.
 */
Graph::Graph()
{
	emax = 0;
	vmax = 0;
	n = 0;
	m = 0;
	head = NULL;
	adjlist = NULL;
	vfirst = NULL;
	reverse = NULL;

	targetVattr = NULL;
	targetMask = 0;
	targetFlag = 0;
	v_label = NULL;

}
/**
 *	free memory depending on the number of edges.
 */
void Graph::freeMemoryE()
{
	if(head!=NULL)		delete [] head;
	if(adjlist!=NULL)	delete [] adjlist;
	head = NULL;
	adjlist = NULL;
}
/**
 *	allocate memory depending on the number of edges.
 */
void Graph::allocMemoryE()
{
	head = new int[emax*2+1];
	adjlist = new int[emax*2+1];
}
/**
 *	free memory depending on the number of vertices.
 */
void Graph::freeMemoryV()
{
	if(vfirst!=NULL)	delete [] vfirst;
	if(reverse!=NULL)	delete [] reverse;
	vfirst = NULL;
	reverse = NULL;
}
/**
 *	allocate memory depending on the number of vertices.
 */
void Graph::allocMemoryV()
{
	vfirst = new int[vmax+1];
	reverse = new int[vmax+1];
}

/**
 *	destructor
 */
Graph::~Graph()
{
	freeMemoryE();
	freeMemoryV();
}

/**
 *	initialize the graph.
 *	@param v maximum of the number of vertices
 *	@param e maximum of the number of edges
 */
void Graph::init(int v, int e)
{
	if(e!=emax){			// reallocate memory if the size has to be changed
		emax = e;
		freeMemoryE();
		allocMemoryE();
	}
	if(v!=vmax){
		vmax = v;
		freeMemoryV();
		allocMemoryV();
	}
	n = 0;
	m = 0;
}

/**
 *	specify the number of vertices and edges actually used.
 *	@param nn the number of vertices actually used.
 *	@param mm the number of edges actually used.
 */
void Graph::setNM(int nn, int mm)
{
	n = nn;
	m = mm;
}
/**
 *	read graph from standard input (cin)
 */
void Graph::graph_input()
{
	cout << "number of vertices n? number of edges m?\n";
	cin >> n >> m;
	if(n > vmax || m > emax){
		int v = (n > vmax) ? n : vmax;
		int e = (m > emax) ? m : emax;
		init(v, e);
	}
	cout << "input tail and head of an edge m times\n";
	for(int a = 1; a <= m; a++){
		int v1, v2;
		cin >> v1 >> v2;
		head[2*a-1]=v1;
		head[2*a]=v2;
	}
}

/**
 *	create adjacency list from information of edges written in head array
 */
void Graph::adjacency_list()
{
	int v;
	for(v = 1; v <= n; v++){		// initialize
		vfirst[v] = 0;
		reverse[v] = 0;
	}
	for(int a = m; a > 0; a--){			// for all edges, 
		int v1 = head[2*a - 1];			// start point	(tail)
		int v2 = head[2*a];				// end point	(head)
		adjlist[2*a] = vfirst[v1];		// add the edge in adjlist
		vfirst[v1] = 2*a;				// improve the top of the list
		adjlist[2*a-1] = reverse[v2];	// add the edge in reverse list
		reverse[v2] = 2*a-1;			// improve the top of the list
	}
}
/**
 *	output the adjacency list (to cout)
 */
void Graph::print_adjlist()
{
	cout << "adjacency list\n";
	for(int v = 1; v <= n; v++){		// vertices
		cout << setw(6) << v << ": ";
		int a = vfirst[v];				// the top edge of the list
		int k = 0;
		while(a!=0){					// till the end of the list
			cout << setw(6) << head[a];
			k++;
			if((k%10)==0) {
				cout << "\n        ";
			}
			a = adjlist[a];				// next edge
		}
		cout<<"\n";
	}
	cout<<"\n";
	cout << "reverse list\n";
	for(int v = 1; v <= n; v++){		// reverse list
		cout << setw(6) << v << ": ";
		int a = reverse[v];
		int k = 0;
		while(a!=0){
			cout << setw(6) << head[a];
			k++;
			if((k%10)==0) {
				cout << "\n        ";
			}
			a = adjlist[a];
		}
		cout<<"\n";
	}
}

/**
 *	set flag on vertices of this graph.
 *	Doing depth-first-search from vertices having mask, 
 *	this sets flag on all vertices reached.
 *	@param revflag if true, the search is done in reverse direction.
 *	@param vattr an array in which flags of each vertex are stored.
 *	@param mask the search is started from vertices having mask
 *	@param flag a flag to be set on vertices
 */
void Graph::searchAndMark(bool revflag, int *vattr, int mask, int flag)
{
	targetVattr = vattr;		// setting the variables
	targetMask = mask;
	targetFlag = flag;
	v_label = new int[n+1];		// allocate memory for label
	if(revflag){
		searchAndMarkRev();		// reverse direction
	} else {
		searchAndMarkFor();
	}
	delete [] v_label;			// free the memory
	v_label = NULL;	
	targetVattr = NULL;
	targetMask = 0;
	targetFlag = 0;
}

/**
 *	depth first search for setting flag
 */
void Graph::searchAndMarkFor()
{
	for(int i = 1; i <= n; i++){	// intialize the label
		v_label[i] = 0;
	}
	for(int i = 1; i <= n; i++){
		if((targetVattr[i]&targetMask)!=0){		// unreached vertex having mask
			searchAndMarkForSub(i);				// start DFS
		}
	}
}
/**
 *	depth first search for setting flag (in reverse direction)
 */
void Graph::searchAndMarkRev()
{
	for(int i = 1; i <= n; i++){
		v_label[i] = 0;
	}
	for(int i = 1; i <= n; i++){
		if((targetVattr[i]&targetMask)!=0){
			searchAndMarkRevSub(i);
		}
	}
}
/**
 *	recursive part of DFS for setting flag (in reverse direction)
 *	@param i id of vertex from which searching should be started.
 */
void Graph::searchAndMarkRevSub(int i)
{
	if(v_label[i]!=0) return;		// return if it has been already reached
	v_label[i] = 1;							// label of arriving
	targetVattr[i] |= targetFlag;			// set the flag
	int a;
	a = reverse[i];							// recursive call w.r.t. all edges
	while(a!=0){
		searchAndMarkRevSub(head[a]);
		a = adjlist[a];
	}
}
/**
 *	recursive part of DFS for setting flag
 *	@param i id of vertex from which searching should be started.
 */
void Graph::searchAndMarkForSub(int i)
{
	if(v_label[i]!=0) return;
	v_label[i] = 1;
	targetVattr[i] |= targetFlag;
	int a;
	a = vfirst[i];
	while(a!=0){
		searchAndMarkForSub(head[a]);
		a = adjlist[a];
	}
}

