/*
 * Graph.hpp	(CCFlib)
 * Copyright (C) 2004 Kento EMOTO
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Kento EMOTO:
 *   mailto:emoto@ipl.t.u-tokyo.ac.jp
 *   http://www.ipl.t.u-tokyo.ac.jp/~emoto/
 */
/** 
 *	@file Graph.hpp definition of class Graph.
 *	@author Kento EMOTO
 */
#ifndef _GRAPH_HPP_
#define _GRAPH_HPP_

#include <iostream>
#include <iomanip>

using namespace std;

/**
 *	a class of directed Graph.
 *	sample code:
 *	<pre>
 *	Graph g;                // create an instance of Graph
 *	g.init(vmax, emax);     // specify max of the number of vertices and edges
 *	for(int k = 1; ...){    // input information of edges
 *		head[k*2-1] = ...   // set the tail of edge k
 *		head[k*2] = ...     // set the head of edge k
 *	}
 *	g.setNM(v, e);          // specify the size of graph
 *	g.adjacency_list();     // create adjacency list of graph
 *	</pre>
 *	@author Kento EMOTO
 */
class Graph {
public:
	int vmax;					/**< maximum of the number of vertices */
	int emax;					/**< maximum of the number of edges */
	int n;						/**< the number of vertices actually used */
	int m;						/**< the number of edges actually used */
	
	/** start and end points of edges.
	 * start points in odd addresses, end points in even addresses.
	 * i.e. an edge e is starting from head[2*e-1] and reaching to head[2*e],
	 * or the tail of an edge is head[2*e-1] and the head is head[2*e].
	 */
	int *head;
	/** adjacency list holding pointer to next edge.
	 * odd addresses for normal direction, even addresses for reverse direction
	 * 
	 * for normal direction, 
	 * all edges starting from vertex v are available by following code.
	 * <pre>
	 *	int a = vfirst[v];    // the top edge of the list (a is an even number)
	 *	while(a!=0){          // till the end of the list
	 *		// do something 
	 *		a = adjlist[a];   // next edge
	 *	}
	 * </pre>
	 *
	 * for reverse direction, 
	 * all edges starting from vertex v are available by following code.
	 * <pre>
	 *	int a = reverse[v];   // top edge of reverse list (a is an odd number)
	 *	while(a!=0){          // till the end of the list
	 *		// do something 
	 *		a = adjlist[a];   // next edge
	 *	}
	 * </pre>
	 * @see Graph#vfirst
	 * @see Graph#reverse
	 */
	int *adjlist;
	/** first edges of each vertex.
	 * edges listed in the list starting from vfirst[v] are 
	 * starting from vertex v.
	 * @see Graph#adjlist
	 */
	int *vfirst;
	/** first edges of each vertex (reverse direction).
	 * edges listed in the list starting from reverse[v] are 
	 * reaching to vertex v.
	 * @see Graph#adjlist
	 */
	int *reverse;
	 
	int *targetVattr;			/**< flags of each vertex fo DFS */
	int targetMask;				/**< flag indicating DFS should be started */
	int targetFlag;				/**< flag to be set by DFS */
	int *v_label;				/**< labels for DFS*/

	Graph();
	~Graph();
	void init(int v, int e);
	void setNM(int nn, int mm);
	void graph_input();
	void adjacency_list();
	void print_adjlist();
	void searchAndMark(bool revflag, int *vattr, int mask, int flag);
				
protected:
	void freeMemoryE();
	void allocMemoryE();
	void freeMemoryV();
	void allocMemoryV();
	void searchAndMarkRev();
	void searchAndMarkFor();
	void searchAndMarkRevSub(int i);
	void searchAndMarkForSub(int i);
};

#endif
