/*
 * MixedMatrix.hpp	(CCFlib)
 * Copyright (C) 2004 Kento EMOTO
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Kento EMOTO:
 *   mailto:emoto@ipl.t.u-tokyo.ac.jp
 *   http://www.ipl.t.u-tokyo.ac.jp/~emoto/
 */
/** 
 *	@file MixedMatrix.hpp definition of class MixedMatrix.
 *	@author Kento EMOTO
 */
#ifndef _MIXEDMATRIX_HPP_
#define _MIXEDMATRIX_HPP_

#include <iostream>
#include <iomanip>
#include "Variable.hpp"					// independent parameter
#include "VariableMatrix.hpp"			// matrix of independent parameters

#include "RealModularNumber.hpp"
#include "RealModularMatrix.hpp"

#include "RationalNumber.hpp"			// rational number
#include "RationalMatrix.hpp"			// matrix of rational numbers
#include "RealNumber.hpp"				// real number
#include "RealMatrix.hpp"				// matrix of real numbers


using namespace std;

/**
 *	a class for mixed matrix.
 *	This holds a mixed matrix and calculates CCF of it. The result is held by this.
 *  The algorith for CCF is based on chapter 2 of "d (): U\ƃASY I (ߑȊw, 1992.)"
 *
 *  CCF = CCF_Q + CCF_T = Pr * S * (Q + T) * Pc
 *
 * sample code (reading from standard in):
 * <pre>
 *	MixedMatrix mm;                      // create an instance
 *	mm.readFromStream(cin);             // read a mixed amtrix form standard in
 *	mm.writeToStream(cout);                // print the matrix to standard out
 *	mm.step1();                            // STEP1(initialize)
 *	mm.step2(true);                        // STEP2(calculate CCF)
 *	cout << "rank = " << mm.rank << "\n";  // rank of matrix
 *	mm.writeToStreamCCF(cout);             // print CCF 
 * </pre>
 * sample code (manually put elements into Q and T from stream):
 * <pre>
 *	istream in;                          // some input stream
 *	MixedMatrix mm;                      // create an instance
 *	mm.init(rq,rt,c);                    // set the size
 *	for(int i = 0; i < rowQ; i++){       // elements of Q (integer)
 *		for(int j = 0; j < col; j++){
 *			int n;
 *			in >> n;
 *			RationalNumber rn(n);
 *			matQ.put(i, j, rn);       // put an element
 *		}
 *	}
 *	for(int i = 0; i < rowT; i++){       // elements of T (id of params)
 *		for(int j = 0; j < col; j++){
 *			int n;
 *			in >> n;
 *			Variable var(n);          // n = 0 means zero
 *			matT.put(i, j, var);
 *		}
 *	}
 * </pre>
 *
 * define _RATIONAL_ to use multi-precision rational numbers, which provides you precise results. However, it takes much time to calculate CCF. Moreover, it needs GMP(GNU MP), which is a library of multi-precision numbers, to compile source files.
 * @see SCD
 * @see Graph
 *	@author Kento EMOTO
 */
class MixedMatrix {
public:
	int rowQ;							/**< the number of rows of Q */
	int rowT;							/**< the number of rows of T */
	int col;							/**< the number of columns of Q (T) */
	RationalMatrix matQ;				/**< matrix Q */
	VariableMatrix matT;				/**< matrix T */


	RealMatrix matS;					/**< transformation matrix S */
	/** column permutation.
	 * Pc(i, Pc[i]) = 1, otherwise 0 for column permutation matrix Pc
	 */
	int *Pc;						
	/** row permutation.
	 * Pr(Pr[i], i) = 1, otherwise 0 for row permutation matrix Pr
	 */
	int *Pr;
	/** column block structure. 
	 *  Cs[0] and Rs[0] are the number of columns and rows of holizontal tail.
	 *  Cs[blocknum+1] and Rs[blocknum+1] are the number of columns and rows of vertical tail.
	 *  Cs[i] and Rs[i] are the number of columns and rows of ith non-singular block (1 <= i <= blocknum).
	 *
	 * @see MixedMatrix#blocknum
	 * @see MixedMatrix#Rs
	 */
	int *Cs;
	/** row block structure. 
	 * @see MixedMatrix#Cs
	 */						
	int *Rs;						
	RealMatrix matCCFQ;			/**< CCF Q part */
	VariableMatrix matCCFT;			/**< CCF T part */

	int blocknum;				/**< the number of non-singular blocks of CCF*/
	int rank;					/**< rank of the matrix */

	MixedMatrix();
	void init(int rq, int rt, int c);
	void readFromStream(istream &in);
	void writeToStream(ostream &out);
	void step1();
	void step2(bool CCFflag);
	~MixedMatrix();
	void printall();
	void printvertex(int v);
	void writeMatrixToStream(ostream &out, RationalMatrix &mat);
	void writeMatrixToStream(ostream &out, RealMatrix &mat);
	void writeToStreamCCF(ostream &out);

	/** return the threshold.
	 * @see MixedMatrix#setThreshold(double t)
	 */
	double getThreshold() { return doubleThreshold; }
	/** set the threshold. 
	 * Each real number obtained by elimination is treated as zero 
	 * if its absolute value is smaller than the threshild set by this method.
	 * Setting this value to zero disables this feature.
	 * @see MixedMatrix#getThreshold()
	 */
	void setThreshold(double t) { doubleThreshold = t; }
protected:
	const static int inI = 0x01;	/**< flag indicating an element of set I */
	const static int inJ = 0x02;		/**< of set J */
	const static int inSminus = 0x04;	/**< of set S- */
	const static int inSplusT = 0x08;	/**< of set S+T */
	const static int inSplusQ = 0x10;	/**< of set S+Q */
	const static int inG0 = 0x20;		/**< of set G0 */
	const static int inGinf = 0x40;		/**< of set Ginf */
	const static int theBTC = 0x01;		/**< edge of BT BC */
	const static int theBP = 0x02;		/**< edge of BP */
	const static int theM = 0x04;		/**< edge of M */
	int flags[3];						/**< theBTC, theBP, theM */
	RealMatrix matP;					/**< eliminated matrix P */
	int *base;							/**< bases of P (making rank) */

	/** information of edges of BT and BC
	 * @see MixedMatrix#adjacency_list(int *vfirst, int *head, int *adjlist, int m, int n)
	 */
	int *BTChead;
	/** adjacency list of BT and BC
	 * @see MixedMatrix#adjacency_list(int *vfirst, int *head, int *adjlist, int m, int n)
	 */
	int *BTCadjlist;
	/** the top of adjacency list of BT and BC
	 * @see MixedMatrix#adjacency_list(int *vfirst, int *head, int *adjlist, int m, int n)
	 */
	int *BTCvfirst;

	/** information of edges of BP
	 * @see MixedMatrix#adjacency_list(int *vfirst, int *head, int *adjlist, int m, int n)
	 */
	int *BPhead;
	/** information of edges of BP
	 * @see MixedMatrix#adjacency_list(int *vfirst, int *head, int *adjlist, int m, int n)
	 */
	int *BPadjlist;
	/** information of edges of BP
	 * @see MixedMatrix#adjacency_list(int *vfirst, int *head, int *adjlist, int m, int n)
	 */
	int *BPvfirst;

	/** information of edges of M
	 * @see MixedMatrix#adjacency_list(int *vfirst, int *head, int *adjlist, int m, int n)
	 */
	int *Mhead;
	/** information of edges of M
	 * @see MixedMatrix#adjacency_list(int *vfirst, int *head, int *adjlist, int m, int n)
	 */
	int *Madjlist;
	/** information of edges of M
	 * @see MixedMatrix#adjacency_list(int *vfirst, int *head, int *adjlist, int m, int n)
	 */
	int *Mvfirst;
	int *vattr;						/**< attributes of vertices */
	int *pathL;						/**< edges of path seached in step2 */
	int *pathLinfo;					/**< information of found path */
	int *v_label;					/**< visited flag */
	int *v_info;					/**< kinds of edges used */
	int *v_que1;					/**< que for WFS */
	int **heads;					/**< BTC, BP, M head */
	int **adjlists;					/**< BTC, BP, M adjlist */
	int **vfirsts;					/**< BTC, BP, M vfirst */
	/** threshold for real number.
	 * @see MixedMatrix#setThreshold(double t)
	 */
	double doubleThreshold;			
	int vnum;					/**< the number of edges of vertices of graph*/
	int BTCnum;					/**< the number of edges of BT and BC */
	int BCnum;					/**< the number of edges of BC */
	int BTnum;					/**< maximum of the number of edges of BT */
	int BPnum;					/**< the number of edges of BP */
	int BPmax;					/**< maximum of the number of edges of BP */
	int Mnum;					/**< the number of edges of M  */
	int Mmax;					/**< maximum of the number of edges of M */
	int pathLnum;				/**< the length of a found path */
	void step4();
	void allocMemory();
	void freeMemory();
	void freeMemoryCCF();
	void allocMemoryCCF();
	void adjacency_list(int *vfirst, int *head, int *adjlist, int m, int n);
	void print_adjlist(int *vfirst, int *head, int *adjlist, int n);
	bool path_search();
	
	/** eliminated matrix P of RealModular Number */
	RealModularMatrix matPM;
	RealModularMatrix matSM;
	/** index of prime numbers used */
	int idxOfPrimesUsed;

	int *base_back;							/**< backup of base */
	int *Mhead_back;						/**< backup of Mhead */

	
	bool reEliminationRealModular();
	void sortBasesByPc(int *bases, int *tmp, int s, int e);
	void duplicateRealModularMatrixFromRationalMatrix(RealModularMatrix &mm, RationalMatrix &rm);
	void step1_forM();
	void step2_updatingM();
	void step3M();
	void allocMemoryForRealModular();
	void freeMemoryForRealModular();
	void writeMatrixToStream(ostream &out,RealModularMatrix &mat);

	bool reEliminationRealModularUpperTriangularByPc();
	void duplicateRealMatrixFromRationalMatrix(RealMatrix &mm, RationalMatrix &rm);

	double *BPweight;			/**< weight of BP edges */
	double *BTCweight;			/**< weight of BTC edges */
	double *weight;				/**< weight of vertices */
	int *v_lastvs;				/**< vertex visited last */
	int *v_layer;				/**< layer of vertices */

	int search_one(int i, int layer, int *que, int *n, double w);



};


#endif
