/*
 * RealModularMatrix.hpp	(CCFlib)
 * Copyright (C) 2004 Kento EMOTO
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Kento EMOTO:
 *   mailto:emoto@ipl.t.u-tokyo.ac.jp
 *   http://www.ipl.t.u-tokyo.ac.jp/~emoto/
 */
/** 
 *	@file RealModularMatrix.hpp implementation and definition of class RealModularMatrix.
 *	@author Kento EMOTO
 */
#ifndef _REALMODULERMATRIX_HPP_
#define _REALMODULERMATRIX_HPP_
#include "RealModularNumber.hpp"

/** matrix of multi-precision rational numbers.
 * (i, j) - element of this matrix is stored in ele[i*col+j] (0 <= i < row, 0<= j < col).
 * @see RealModularNumber
 *	@author Kento EMOTO
 */
class RealModularMatrix {
public:
	RealModularNumber *ele;			/**< elements (i, j) -> [i*col+j] */
	int col;						/**< the number of columns */
	int row;						/**< the number of rows */
	
	/** 
	 *	constructor: initialized as 0x0 matrix
	 */
	RealModularMatrix()
	{
		row = 0;
		col = 0;
		ele = NULL;
	}
	/** 
	 *	constructor: initialized as rxc matrix
	 *	@param r the number of rows
	 *	@param c the number of columns
	 */
	RealModularMatrix(int r, int c)
	{
		row = r;
		col = c;
		ele = NULL;
		allocMemory();
	}
	/** destructor */
	~RealModularMatrix()
	{
		freeMemory();
	}
	/** 
	 *	initialize this matrix as rxc matrix
	 *	@param r the number of rows
	 *	@param c the number of columns
	 */
	void init(int r, int c){
		row = r;
		col = c;
		freeMemory();
		allocMemory();
	}
	/** free memory */
	void freeMemory()
	{
		if(ele!=NULL) delete [] ele;
		ele = NULL;
	}
	/** allocate memory for elements */
	void allocMemory()
	{
		ele = new RealModularNumber[row*col];
	}
	/**
	 * get the elemtent of (i, j).
	 *	@param i row of the element (0 <= i < row)
	 *	@param j column of the element (0 <= j < col)
	 */
	RealModularNumber &get(int i, int j){
		return ele[i*col + j];
	}
	/** 
	 * put an elemtent to (i, j).
	 *	@param i row of the element (0 <= i < row)
	 *	@param j column of the element (0 <= j < col)
	 *	@param dat RealModularNumber to be put to (i, j)
	 */
	void put(int i, int j, RealModularNumber &dat){
		ele[i*col + j] = dat;
	}
	/**
	 *	duplicate from another matrix.
	 *	@param rm another matrix which this matrix duplicates
	 */
	void copyFrom(const RealModularMatrix &rm){
		init(rm.row, rm.col);
		int rc = col*row;
		for(int i = 0; i < rc; i++){
			ele[i] = rm.ele[i];
		}
	}
	/** 
	 * make this an identity matrix of rxr.
	 *	@param r row of the matrix
	 */
	void setIdentity(int r){
		init(r, r);
		RealModularNumber rnOne(1);
		for(int i = 0; i < r; i++){
			put(i,i,rnOne);
		}
	}
};
#endif
