/*
 * RealModularNumber.hpp	(CCFlib)
 * Copyright (C) 2004 Kento EMOTO
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Kento EMOTO:
 *   mailto:emoto@ipl.t.u-tokyo.ac.jp
 *   http://www.ipl.t.u-tokyo.ac.jp/~emoto/
 */
/** 
 *	@file RealModularNumber.hpp implementation and definition of class RealModularNumber.
 *	@author Kento EMOTO
 */
#ifndef _MODULERNUMBER_HPP_
#define _MODULERNUMBER_HPP_

#include <iostream>
#include <iomanip>

using namespace std;
#include <stdlib.h>
#include <time.h>

extern int thePrime;		/**< prime used by RealModularNumber */

extern double eps;

void setEps(double e);
double getEps();

int setPrime(int p);

extern int exp_id;
/**
 *	an exception thrown when the prime is not luckey.
 */
class ThePrimeIsNotLuckeyException {
public:
	int e;
	ThePrimeIsNotLuckeyException(){e = exp_id++;}
	/**
	 *	stream output operator(<<).
	 */
	friend ostream &operator<<(ostream &stream, ThePrimeIsNotLuckeyException &exp){
		stream << "ThePrimeIsNotLuckeyException(" << exp.e << ")";
		return stream;
	}
};

/**
 *	number for moduler calculus
 */
class RealModularNumber {
public:
	/**
	 *	number in Z_p.
	 *	num should be normalized as 0 <= num < thePrime 
	 *	or -thePrime/2 <= num < thePrime/2
	 */
	int num;
	/**
	 *	floating point representation.
	 */
	 double val;

	/**
	 *	destructor.
	 */
	~RealModularNumber(){
	}
	/**
	 *	constructor: 0.
	 */
	RealModularNumber(){
		num = 0;
		val = 0;
	}
	/**
	 *	constructor: Integer n.
	 */
	RealModularNumber(int n){
		num = n % thePrime;
		val = n;
	}
	/**
	 *	constructor: Rational n/d.
	 */
	RealModularNumber(int n, int d){
		int inv = getInverse(d, thePrime);
		num = (inv * (long long)n) % thePrime;
		val = n/(double)d;
	}
	/**
	 *	copy constructor.
	 */
	RealModularNumber(const RealModularNumber &mn){
		num = mn.num;
		val = mn.val;
	}
	/**
	 *	addition operator(+).
	 */
	RealModularNumber operator+(const RealModularNumber &mn){
		RealModularNumber mn2;
		mn2.num = (num + (long long)mn.num)%thePrime;
		if(mn2.num < 0) mn2.num += thePrime;
		mn2.val = val + mn.val;
		if(mn2.num==0){
			double d = abs(mn2.val);
			if((d/thePrime) > eps){
				throw ThePrimeIsNotLuckeyException();
			}
			mn2.val = 0;
		}
		return mn2;
	}
	/**
	 *	subtraction operator(-).
	 */
	RealModularNumber operator-(const RealModularNumber &mn){
		RealModularNumber mn2;
		mn2.num = (num - (long long)mn.num)%thePrime;
		if(mn2.num < 0) mn2.num += thePrime;
		mn2.val = val - mn.val;
		if(mn2.num==0){
			double d = abs(mn2.val);
			if((d/thePrime) > eps){
				throw ThePrimeIsNotLuckeyException();
			}
			mn2.val = 0;
		}
		return mn2;
	}

	/**
	 *	multiplicatoin operator(*).
	 */
	RealModularNumber operator*(const RealModularNumber &mn){
		RealModularNumber mn2;
		mn2.num = (num * (long long)mn.num)%thePrime;
		if(mn2.num < 0) mn2.num += thePrime;
		mn2.val = val * mn.val;
		if(mn2.num==0){
			double d = abs(mn2.val);
			if((d/thePrime) > eps){
				throw ThePrimeIsNotLuckeyException();
			}
			mn2.val = 0;
		}
		return mn2;
	}

	/**
	 *	division operator(/).
	 */
	RealModularNumber operator/(const RealModularNumber &mn){
		RealModularNumber mn2;
		int inv = getInverse(mn.num, thePrime);
		mn2.num = (num * (long long)inv) % thePrime;
		if(mn2.num < 0) mn2.num += thePrime;
		mn2.val = val / mn.val;
		if(mn2.num==0){
			double d = abs(mn2.val);
			if((d/thePrime) > eps){
				throw ThePrimeIsNotLuckeyException();
			}
			mn2.val = 0;
		}
		return mn2;
	}

	/**
	 *	negation operator(-).
	 */
	RealModularNumber operator-(){
		RealModularNumber mn2;
		mn2.num = (- num) % thePrime;
		if(mn2.num < 0) mn2.num += thePrime;
		mn2.val = - val;
		return mn2;
	}

	/**
	 *	substitution operator(=).
	 */
	RealModularNumber &operator=(const RealModularNumber &mn){
		num = mn.num;
		val = mn.val;
		return *this;
	}
	/**
	 *	stream output operator(<<).
	 */
	friend ostream &operator<<(ostream &stream, RealModularNumber &mn){
		stream << "[";
		stream << mn.num;
		stream << "]";
		stream << "(" << mn.val << ")";
		return stream;
	}
	/**
	 *	return true if this is zero on all fields('Z_p's).
	 */
	bool isZero(){
		if(num!=0) return false;
		val = 0;
		return true;
	}
	/**
	 *	return true if this is not zero on all fields('Z_p's).
	 */
	bool isNotZero(){
		if(num==0) return false;
		return true;
	}

protected:
	/**
	 *	get inverse of n in Z_p.
	 *	i.e. integer m satisfying n * m + p * l = 1
	 */
	int getInverse(int n, int p){
		if(n==0){
			cout << "divide by zero? in RealModularNumber::getInverse(p = " << p << ")\n";
		}
		int sn = 1;
		if(n < 0){
			sn = -1;
			n = -n;
		}
		int a0 = 0;
		int a1 = 1;
		int b0 = 1;
		int b1 = 0;
		int r0 = p;
		int r1 = n;
		int d;
		int tmp;
		while(r1!=0){
			d = r0 / r1;
			tmp = r0 % r1;
			r0 = r1;
			r1 = tmp;
			tmp = a0 - d * a1;
			a0 = a1;
			a1 = tmp;
			tmp = b0 - d * b1;
			b0 = b1;
			b1 = tmp;
		}
		
		return (a0 * sn);
	}
};

#endif
