/*
 * bmp.cpp	(CCFlib)
 * Copyright (C) 2004 Kento EMOTO
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Kento EMOTO:
 *   mailto:emoto@ipl.t.u-tokyo.ac.jp
 *   http://www.ipl.t.u-tokyo.ac.jp/~emoto/
 */
/** 
 *	@file bmp.cpp implementation of functions of BMPManager.
 *	@author Kento EMOTO
 */




#include "bmp.hpp"


/**
 *	initialize BMP
 *	@param ww width of BMP (ww > 0)
 *	@param hh height of BMP (hh > 0)
 */
void BMPManager::init(int ww, int hh)
{
	freeMemory();
	w = ww;
	h = hh;
	wsize = ((((w * 24) + 31) & ~31) >> 3);
	allocMemory();
	int size = wsize * h;
	bmpheader->bfType      = 0x4d42;
	bmpheader->bfSize      = size + 54;
	bmpheader->bfReserved1 = 0;
	bmpheader->bfReserved2 = 0;
	bmpheader->bfOffBits   = 54;
	dibheader->biSize          = 40;
	dibheader->biWidth         = w;
	dibheader->biHeight        = h;
	dibheader->biPlanes        = 1;
	dibheader->biBitCount      = 24;
	dibheader->biCompression   = 0;
	dibheader->biSizeImage     = size;
	dibheader->biXPelsPerMeter = 0;
	dibheader->biYPelsPerMeter = 0;
	dibheader->biClrUsed       = 0;
	dibheader->biClrImportant  = 0;
}

/**
 *	free memory
 */
void BMPManager::freeMemory()
{
	if(bmpdata!=NULL) delete [] bmpdata;
	bmpdata = NULL;
	if(bmpheader!=NULL) delete bmpheader;
	bmpheader = NULL;
	if(dibheader!=NULL) delete dibheader;
	dibheader = NULL;
}

/**
 *	allocate memory
 */
void BMPManager::allocMemory()
{
	allocMemory1();
	allocMemory2();
}
/**
 *	allocate memory 1 (for headers)
 */
void BMPManager::allocMemory1()
{
	bmpheader = new BMPHEADER;
	dibheader = new DIBHEADER;
}
/**
 *	allocate memory 2 (for image data)
 */
void BMPManager::allocMemory2()
{
	bmpdata = new unsigned char[wsize*h];
}

/**
 *	constructor: NULL image
 */
BMPManager::BMPManager()
{
	bmpdata = NULL;
	bmpheader = NULL;
	dibheader = NULL;
	w = 0;
	h = 0;
	wsize = 0;
}

/**
 *	destructor
 */
BMPManager::~BMPManager()
{
	freeMemory();
}

/**
 *	output BMP header
 */
void BMPManager::printBMPHeader()
{
	cout << "bfType      = "<< setbase(16) << bmpheader->bfType << "\n";
	cout << setbase(10);
	cout << "bfSize      = " << bmpheader->bfSize << "\n";
	cout << "bfReserved1 = " << bmpheader->bfReserved1 << "\n";
	cout << "bfReserved2 = " << bmpheader->bfReserved2 << "\n";
	cout << "bfOffBits   = " << bmpheader->bfOffBits << "\n";
}
/**
 *	output DIB header
 */
void BMPManager::printDIBHeader()
{
	cout << "biSize          = " << dibheader->biSize << "\n";
	cout << "biWidth         = " << dibheader->biWidth << "\n";
	cout << "biHeight        = " << dibheader->biHeight << "\n";
	cout << "biPlanes        = " << dibheader->biPlanes << "\n";
	cout << "biBitCount      = " << dibheader->biBitCount << "\n";
	cout << "biCompression   = " << dibheader->biCompression << "\n";
	cout << "biSizeImage     = " << dibheader->biSizeImage << "\n";
	cout << "biXPelsPerMeter = " << dibheader->biXPelsPerMeter << "\n";
	cout << "biYPelsPerMeter = " << dibheader->biYPelsPerMeter << "\n";
	cout << "biClrUsed       = " << dibheader->biClrUsed << "\n";
	cout << "biClrImportant  = " << dibheader->biClrImportant << "\n";
}

/**
 *	read BMP from input stream
 *	@param in input byte-stream (usually file stream)
 */
void BMPManager::loadBMP(istream &in)
{
	freeMemory();
	allocMemory1();
	in.read((char *)bmpheader, sizeof(BMPHEADER));
	in.read((char *)dibheader, sizeof(DIBHEADER));
	if(dibheader->biBitCount != 24){
		cout << "cannot read non full-color BMP\n";
		freeMemory();
		return;
	}
	w = dibheader->biWidth;
	h = dibheader->biHeight;
	wsize = ((((w * 24) + 31) & ~31) >> 3);
	allocMemory2();
	for(int i = h-1; i>=0; i--){
		in.read((char *)(&(bmpdata[i*wsize])), wsize);
	}
}

/**
 *	write BMP to output stream
 *	@param out output byte-stream (usually file stream)
 */
void BMPManager::saveBMP(ostream &out)
{
	out.write((char *)bmpheader, sizeof(BMPHEADER));
	out.write((char *)dibheader, sizeof(DIBHEADER));
	for(int i = h-1; i>=0; i--){
		out.write((char *)(&(bmpdata[i*wsize])), wsize);
	}
}

