/*
 * bmp.hpp	(CCFlib)
 * Copyright (C) 2004 Kento EMOTO
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Kento EMOTO:
 *   mailto:emoto@ipl.t.u-tokyo.ac.jp
 *   http://www.ipl.t.u-tokyo.ac.jp/~emoto/
 */
/** 
 *	@file bmp.hpp definition of class BMPManager.
 *	@author Kento EMOTO
 */

#ifndef _BMP_HPP_
#define _BMP_HPP_

#include <iostream>
#include <fstream>
#include <iomanip>

using namespace std;

#pragma pack (push, 1)
/**
 *	structure of BMP header
 */
typedef struct _BMPHEADER 
{
	short bfType;			/**< "BM" */
	int bfSize;				/**< size of BMP file*/
	short bfReserved1;		/**< reserved */
	short bfReserved2;		/**< reserved */
	int bfOffBits;			/**< offset address of image data */
} BMPHEADER;

/**
 *	structure of DIB header
 */
typedef struct _DIBHEADER 
{
	int biSize;				/**< size of DIB header */
	int biWidth;			/**< width of image */
	int biHeight;			/**< height of image */
	short biPlanes;			/**< width of image */
	short biBitCount;		/**< bits per pixel */
	int biCompression;		/**< flag of compression */
	int biSizeImage;		/**< size of image data */
	int biXPelsPerMeter;	/**< resolution */
	int biYPelsPerMeter;	/**< resolution */
	int biClrUsed;			/**< the number of colors used */
	int biClrImportant;		/**< the number of important colors */
} DIBHEADER;
#pragma pack (pop)

/**
 *	wrapper for BMP manipulation (24bit/pixel only).
 *	usage:
 *	- make an instance of this class
 *	- load BMP from an input stream in using loadBMP(in)
 *	- initialize size of BMP by using init(ww, hh) and set color of pixels by using setRGB(x, y, r, g, b)
 *	- save BMP to an output stream out using saveBMP(out)
 *	.
 *	sample code for load
 *	<pre>
 *	BMPManager bmp;                              // create an instance
 *	ifstream in;                                 // create a file input stream
 *	in.open("test.bmp", ios::in | ios::binary);  // open the stream
 *	bmp.loadBMP(in);                             // load BMP
 *	in.close();                                  // close the stream
 *	</pre>
 *	sample code for save
 *	<pre>
 *	BMPManager bmp;                              // create an instance
 *	bmp.init(640,480);                           // initialize size of image
 *	for(int x = 0; x < 640; x++){                // set color of pixels
 *		for(int y = 0; y < 480; y++){
 *			bmp.setRGB(x, y, (int)(255*(640-x)/640.), (int)(255*(480-y)/480.), 255);
 *		}
 *	}
 *	ofstream out;                                // create a file output stream
 *	out.open("test2.bmp", ios::out | ios::binary); // open the stream
 *	bmp.saveBMP(out);                            // save BMP
 *	out.close();                                 // close the stream
 *	</pre>
 *	@author Kento EMOTO
 */
class BMPManager{
public:
	void loadBMP(istream &in);
	void saveBMP(ostream &out);
	BMPManager();
	~BMPManager();
	void init(int ww, int hh);
	void printBMPHeader();
	void printDIBHeader();

	/**
	 *	set color of a pixel at (x, y).
	 *	@param x x-coordinates (0 <= x < width)
	 *	@param y y-coordinates (0 <= y < height)
	 *	@param r red (0 <= r < 256)
	 *	@param g green (0 <= g < 256)
	 *	@param b blue (0 <= b < 256)
	 */
	void setRGB(int x, int y, unsigned char r, unsigned char g, unsigned char b)
	{
		int addr = x * 3 + y * wsize;
		bmpdata[addr] = b;
		bmpdata[addr+1] = g;
		bmpdata[addr+2] = r;
	}
	
	/**
	 *	get color of a pixel at (x, y).
	 *	the color is stored in triplet (*r, *g, *b)
	 *	@param x x-coordinates (0 <= x < width)
	 *	@param y y-coordinates (0 <= y < height)
	 *	@param r red (0 <= r < 256)
	 *	@param g green (0 <= g < 256)
	 *	@param b blue (0 <= b < 256)
	 */
	void getRGB(int x, int y, unsigned char *r, unsigned char *g, unsigned char *b)
	{
		int addr = x * 3 + y * wsize;
		*b = bmpdata[addr];
		*g = bmpdata[addr+1];
		*r = bmpdata[addr+2];
	}

private:
	void allocMemory();
	void allocMemory1();
	void allocMemory2();
	void freeMemory();

	/**
	 *	BMP header
	 */
	BMPHEADER *bmpheader;
	/**
	 *	DIB header
	 */
	DIBHEADER *dibheader;
	/**
	 *	BMP data.
	 *	color of a pixel at (x, y):<br>
	 *	addr = x * 3 + y * wsize  (0 <= x < width, 0 <= y < height)
	 *	- bmpdata[addr  ] :	blue
	 *	- bmpdata[addr+1] :	green
	 *	- bmpdata[addr+2] :	red
	 *	.
	 */
	unsigned char *bmpdata;
	/**
	 *	width of BMP.
	 */
	int w;
	/**
	 *	height of BMP.
	 */
	int h;
	/**
	 *	bytes per line.
	 *	wsize = ((((w * 24) + 31) & ~31) >> 3) for full color
	 */
	int wsize;
};

#endif
