/*
 * scd.cpp	(CCFlib)
 * Copyright (C) 2004 Kento EMOTO
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Kento EMOTO:
 *   mailto:emoto@ipl.t.u-tokyo.ac.jp
 *   http://www.ipl.t.u-tokyo.ac.jp/~emoto/
 */
/** 
 *	@file scd.cpp implementation of class SCD.
 *	@author Kento EMOTO
 */
#include "scd.hpp"

/**
 *	constructor.
 *	initially no graph is associated to this instance, 
 *	so a directed graph should be passed by calling init as an instance of Graph class
 *  @see SCD#init(const Graph &g)
 */
SCD::SCD(){
	emax = 0;
	vmax = 0;
	n = 0;
	m = 0;
	head = NULL;
	adjlist = NULL;
	vfirst = NULL;
	reverse = NULL;
	v_label = NULL;
	order = NULL;
	component = NULL;
	vcomp = NULL;
	complist = NULL;
	vcomplist = NULL;
}
/**
 *	initialize by directed graph g
 *	@param g directed graph of which SCD should be calculated
 *	@see Graph
 */
void SCD::init(const Graph &g)
{
	if(emax != g.emax){		// reallocate memory if the size has to be changed
		emax = g.emax;
		freeMemoryE();
		allocMemoryE();
	}
	if(vmax != g.vmax){
		vmax = g.vmax;
		freeMemoryV();
		allocMemoryV();
	}
	m = g.m;				// copy information of the graph
	n = g.n;
	head = g.head;			// copy only pointer for arrays
	adjlist = g.adjlist;
	vfirst = g.vfirst;
	reverse = g.reverse;
}
/**
 *	destructor
 */
SCD::~SCD()
{
	freeMemoryE();
	freeMemoryV();
}
/**
 *	allocate memory depending on vmax
 */
void SCD::allocMemoryV()
{
	v_label = new int [vmax+1];
	order = new int [vmax+1];
	component = new int [vmax+1];
	vcomp = new int [vmax+1];
	vcomplist = new int [vmax+1];
}
/**
 *	allocate memory depending on emax
 */
void SCD::allocMemoryE()
{
	complist = new int [emax*2+1];
}
/**
 *	free memory depending on vmax
 */
void SCD::freeMemoryV()
{
	if(v_label!=NULL)	delete [] v_label;
	if(order!=NULL)		delete [] order;
	if(component!=NULL)	delete [] component;
	if(vcomp!=NULL)		delete [] vcomp;
	if(vcomplist!=NULL)	delete [] vcomplist;
	v_label = NULL;
	order = NULL;
	component = NULL;
	vcomp = NULL;
	vcomplist = NULL;
}
/**
 *	free memory depending on emax
 */
void SCD::freeMemoryE()
{
	if(complist!=NULL)	delete [] complist;
	complist = NULL;
}

/**
 *	DFS for post order
 */
void SCD::depth_first_search()
{
	int v;
	for(v = 1; v <= n; v++){
		v_label[v] = unvisited;			// initialize label by unvisited
	}
	jdx = 0;
	for(v = 1; v <= n; v++){
		if(v_label[v] == unvisited){	// do DFS if v is unvisited
			dfs(v);
		}
	}
}
/**
 *	recursive part of DFS
 *	@param v vertex from which searching should be started.
 */
void SCD::dfs(int v)
{
	int a, w;
	v_label[v] = 1;				// set flag indicating v has been reached.
	a=vfirst[v];				// the top of the edges starting from v
	while(a!=0){				// for all edges
		w = head[a];
		if(v_label[w] == unvisited) dfs(w);		// search if w is unvisited 
		a = adjlist[a];							// next edge
	}
	jdx++;
	order[jdx] = v;				// register post order
}

/** 
 * obtain strongly-connected component decomposition including vertex v
 * @param v vertex from which the search should be started.
 */
void SCD::stcomp(int v)			
{
	v_label[v] = idx;				// assign a number of scc
	int a = reverse[v];				// by reverse direction
	while(a!=0){
		int w = head[a];
		if(v_label[w]==unvisited){	// do DFS starting from unreached vertex
			if (newflag) {
				newflag = false;
				complist[a] = 0;
			} else {
				complist[a] = top;		// add to the edge list
			}
			top = a;					// the top of the edge list
			vcomplist[w] = vtop;		// add to the vertex list
			vtop = w;					// the top of the vertex list
			stcomp(w);
		} else {
			if(v_label[w]==idx) {		// this edge is in the same scc
				complist[a] = top;
				top = a;
			}
		}
		a = adjlist[a];					// next edge
	}
}
/**
 *	calculate strongly-connected component decomposition of the graph
 */
void SCD::scd()
{
	depth_first_search();			// obtain post order

	int v;
	for(v = 1; v <=n; v++){
		v_label[v] = unvisited;		// initialize the label
	}
	idx = 0;
	top = 0;
			// DFS starting from vertice having larger post order
	for(int kk = n; kk > 0; kk--){
		v = order[kk];
		if(v_label[v] == unvisited){
			idx++;							// next scc
			vcomplist[v] = 0;
			vtop = v;
			newflag = true;
			stcomp(v);		// obtain all strongly-connected components by DFS
			vcomp[idx] = vtop;
			if(newflag){
				component[idx] = 0;		// no edge (that is one vertex)
			} else {
				component[idx] = top;	// head of edge list
			}
		}
	}
}
/**
 *	output the result SCD
 */
void SCD::print()
{
	for(int k = 1; k <= idx; k++){
		int kk = 0;
		cout << "strong component[" << k << "]\n";
		int v = vcomp[k];
		cout << "  vertices:";
		while(v!=0){
			kk++;
			cout << setw(6) << v;
			if((kk%10)==0) cout << "\n";
			v = vcomplist[v];
		}
		cout << "\n";
		int a = component[k];
		while(a!=0){
			cout << " edge = " << setw(4) << (a+1)/2;
			cout << " tail = " << setw(4) << head[a];
			cout << " head = " << setw(4) << head[a+1];
			cout << "\n";
			a = complist[a];
		}
		cout << "\n";
	}
}



