/*
 * scd.hpp	(CCFlib)
 * Copyright (C) 2004 Kento EMOTO
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * Kento EMOTO:
 *   mailto:emoto@ipl.t.u-tokyo.ac.jp
 *   http://www.ipl.t.u-tokyo.ac.jp/~emoto/
 */
/** 
 *	@file scd.hpp definition of class SCD.
 *	@author Kento EMOTO
 */
#ifndef _SCD_HPP_
#define _SCD_HPP_

#include <iostream>
#include <iomanip>

using namespace std;

#include "Graph.hpp"			// Graph class for directed graph

/**
 *	calculate SCD(strongly-connected component decomposition) of a directed graph.
 *	based on algorithm written in "Fv: ̍\ [] ({]_, 1994.)"
 *	This class takes a directed graph as an instance of Graph class.
 *
 *	usage:
 *	- make an instance of this class
 *	- initialize the instance by a directed graph
 *	- call scd() to calculate SCD of the graph
 *	.
 *	sample code:
 *	<pre>
 *	Graph g2;               // create an instance of Graph
 *	g2.init(vmax, emax);    // specify max of the number of vertices and edges
 *	for(int k = 1; ...){    // input information of edges
 *		head[k*2-1] = ...   // set the head of edge k
 *		head[k*2] = ...     // set the tail of edge k
 *	}
 *	g2.setNM(v, e);         // specify the size of graph
 *	g2.adjacency_list();    // create adjacency list of graph
 *	SCD scd;                // create an instance of SCD
 *	scd.init(g2);           // initialize the instance by g2
 *	scd.scd();              // do SCD
 *	</pre>
 *	@see Graph
 *	@author Kento EMOTO
 */
class SCD {
public:
	int idx;			/**< the number of scc */
	int *v_label;		/**< number of scc to which each vertex should belong*/
	int *component;		/**< the top of edge list of scc */
	int *vcomp;			/**< the top of vertex list of scc */
	int *complist;		/**< edge list of scc */
	int *vcomplist;		/**< vertex list of scc */

	SCD();
	~SCD();
	void scd();
	void print();
	void init(const Graph &g);

protected:
	/** flag which indicates that the vertex has not been reached.	 */
	const static int unvisited = 0;		
	int vmax;					/**< maximum of the number of vertices */
	int emax;					/**< maximum of the number of edges */
	int n;						/**< the number of vertices actually used */
	int m;						/**< the number of edges actually used */

	/** start and end points of edges.
	 * @see Graph#head
	 */
	const int *head;
	/** adjacency list holding pointer to next edge.
	 * @see Graph#adjlist
	 */
	const int *adjlist;
	/** first edges of each vertex.
	 * @see Graph#vfirst
	 */
	const int *vfirst;
	/** first edges of each vertex (reverse direction).
	 * @see Graph#reverse
	 */
	const int *reverse;
	int *order;					/**< vertices in post order */
	
	int jdx;					/**< current post order in depth first search*/
	int top;					/**< holding the top edge during SCD */
	int vtop;					/**< holding the top vertex during SCD */
	/** flag indicating whether the edge is first added to the list or not */
	bool newflag;



	void freeMemoryE();
	void allocMemoryE();
	void freeMemoryV();
	void allocMemoryV();
	void depth_first_search();
	void dfs(int w);
	void stcomp(int v);
};
#endif
